@@ -11,7 +11,7 @@ Grokdocs: Build the Grok documentation in different formats!
 """,
     install_requires=['docutils==0.4',
                       'Sphinx==0.4',
-                      'Pygments',
+                      'Pygments==0.8.1',
                       ],
     package_dir = {'': 'src'},
     packages=find_packages('src'),
