@@ -3,9 +3,6 @@ Containers can also have an explicit traverser associated with them.
 The behavior falls back to basic container traversal if the 'traverse'
 method returns None:
 
-  >>> import grok
-  >>> from grok.ftests.traversal.containertraverser import Herd, Mammoth
-  >>> grok.grok('grok.ftests.traversal.containertraverser')
   >>> getRootFolder()["herd"] = herd = Herd()
   >>> herd['manfred'] = Mammoth('Manfred')
   >>> herd['ellie'] = Mammoth('Ellie')
@@ -51,6 +48,14 @@ The fall-back behavior should work for items that aren't traversed:
   </body>
   </html>
 
+Also try traversing (an empty and therefore False in a Boolean sense) container
+as a subitem of a container:
+
+  >>> herd['subherd'] = Herd()
+  >>> browser.open("http://localhost/herd/subherd/special")
+  >>> print browser.contents
+  special view
+
 """
 import grok
 
@@ -63,7 +68,7 @@ class Traverser(grok.Traverser):
         if name == 'special':
             return Special()
         return None
-    
+
 class Mammoth(grok.Model):
     def __init__(self, name):
         self.name = name
