@@ -11,17 +11,24 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""Grokkers for the various components."""
+"""Grokkers for Grok-configured components.
 
+This `meta` module contains the actual grokker mechanisms for which the
+Grok web framework is named.  A directive in the adjacent `meta.zcml`
+file directs the `martian` library to scan this file, where it discovers
+and registers the grokkers you see below.  The grokkers are then active
+and available as `martian` recursively examines the packages and modules
+of a Grok-based web application.
+
+"""
 import zope.component.interface
 from zope import interface, component
+from zope.interface.interface import InterfaceClass
 from zope.publisher.interfaces.browser import (IDefaultBrowserLayer,
-                                               IBrowserRequest,
                                                IBrowserPublisher)
 from zope.publisher.interfaces.http import IHTTPRequest
 
 from zope.publisher.interfaces.xmlrpc import IXMLRPCRequest
-from zope.viewlet.interfaces import IViewletManager, IViewlet
 from zope.securitypolicy.interfaces import IRole
 from zope.securitypolicy.rolepermission import rolePermissionManager
 
@@ -47,13 +54,10 @@ import grok
 from grok import components
 from grok.util import make_checker
 from grok.interfaces import IRESTSkinType
-from grok.interfaces import IViewletManager as IGrokViewletManager
 
-from grokcore.component.scan import determine_module_component
 from grokcore.security.meta import PermissionGrokker
 
-from grokcore.view.meta.views import (
-    default_view_name, default_fallback_to_name)
+from grokcore.view.meta.views import default_fallback_to_name
 
 
 def default_annotation_provides(factory, module, **data):
@@ -67,19 +71,20 @@ def default_annotation_name(factory, module, **data):
     return factory.__module__ + '.' + factory.__name__
 
 
-class ViewletManagerContextGrokker(martian.GlobalGrokker):
-
-    martian.priority(1001)
-
-    def grok(self, name, module, module_info, config, **kw):
-        viewletmanager = determine_module_component(module_info,
-                                                    grok.viewletmanager,
-                                                    IGrokViewletManager)
-        grok.viewletmanager.set(module, viewletmanager)
-        return True
-
-
 class XMLRPCGrokker(martian.MethodGrokker):
+    """Grokker for methods of a `grok.XMLRPC` subclass.
+
+    When an application defines a `grok.XMLRPC` view, we do not actually
+    register the view with the Component Architecture.  Instead, we grok
+    each of its methods separately, placing them each inside of a new
+    class that we create on-the-fly by calling `type()`.  We make each
+    method the `__call__()` method of its new class, since that is how
+    Zope always invokes views.  And it is this new class that is then
+    made the object of the two configuration actions that we schedule:
+    one to activate it as an XML-RPC adapter for the context, and the
+    other to prepare a security check for the adapter.
+
+    """
     martian.component(grok.XMLRPC)
     martian.directive(grok.context)
     martian.directive(grok.require, name='permission')
@@ -109,12 +114,30 @@ class XMLRPCGrokker(martian.MethodGrokker):
 
 
 class RESTGrokker(martian.MethodGrokker):
+    """Grokker for methods of a `grok.REST` subclass.
+
+    When an application defines a `grok.REST` view, we do not actually
+    register the view with the Component Architecture.  Instead, we grok
+    each of its methods separately, placing them each inside of a new
+    class that we create on-the-fly by calling `type()`.  We make each
+    method the `__call__()` method of its new class, since that is how
+    Zope always invokes views.  And it is this new class that is then
+    made the object of the two configuration actions that we schedule:
+    one to activate it as a REST adapter for the context, and the other
+    to prepare a security check for the adapter.
+
+    This results in several registered views, typically with names like
+    `GET`, `PUT`, and `POST` - one for each method that the `grok.REST`
+    subclass defines.
+
+    """
     martian.component(grok.REST)
     martian.directive(grok.context)
     martian.directive(grok.layer, default=grok.IRESTLayer)
     martian.directive(grok.require, name='permission')
 
-    def execute(self, factory, method, config, permission, context, layer, **kw):
+    def execute(self, factory, method, config, permission, context,
+                layer, **kw):
         name = method.__name__
 
         method_view = type(
@@ -136,23 +159,79 @@ class RESTGrokker(martian.MethodGrokker):
         return True
 
 
+_restskin_not_used = object()
+
+class RestskinInterfaceDirectiveGrokker(martian.InstanceGrokker):
+    """Grokker for interfaces providing the `grok.restskin()` directive.
+
+    Applications create REST skins by subclassing `grok.IRESTLayer`
+    and providing the subclass with a `grok.restskin()` directive giving
+    the prefix string which distinguishes that REST layers from others.
+    This grokker registers those skins.
+
+    """
+    martian.component(InterfaceClass)
+
+    def grok(self, name, interface, module_info, config, **kw):
+        # This `InstanceGrokker` will be called for every instance of
+        # `InterfaceClass` - that is, for every interface defined in an
+        # application module!  So we have to do our own filtering, by
+        # checking whether each interface includes the `grok.restskin()`
+        # directive, and skipping those that do not.
+        restskin = grok.restskin.bind(default=_restskin_not_used
+                                      ).get(interface)
+        if restskin is _restskin_not_used:
+            # The restskin directive is not actually used on the found
+            # interface.
+            return False
+
+        if not interface.extends(grok.IRESTLayer):
+            # For REST layers it is required to extend IRESTLayer.
+            raise GrokError(
+                "The grok.restskin() directive is used on interface %r. "
+                "However, %r does not extend IRESTLayer which is "
+                "required for interfaces that are used as layers and are to "
+                "be registered as a restskin."
+                % (interface.__identifier__, interface.__identifier__),
+                interface
+                )
+        config.action(
+            discriminator=('restprotocol', restskin),
+            callable=zope.component.interface.provideInterface,
+            args=(restskin, interface, IRESTSkinType)
+            )
+        return True
+
+
 class JSONGrokker(martian.MethodGrokker):
+    """Grokker for methods of a `grok.JSON` subclass.
+
+    When an application defines a `grok.JSON` view, we do not actually
+    register the view with the Component Architecture.  Instead, we grok
+    each of its methods separately, placing them each inside of a new
+    class that we create on-the-fly by calling `type()`.  We make each
+    method the `__call__()` method of its new class, since that is how
+    Zope always invokes views.  And it is this new class that is then
+    made the object of the two configuration actions that we schedule:
+    one to activate it as a JSON adapter for the context, and the other
+    to prepare a security check for the adapter.
+
+    """
     martian.component(grok.JSON)
     martian.directive(grok.context)
     martian.directive(grok.require, name='permission')
+    martian.directive(grok.layer, default=IDefaultBrowserLayer)
 
-    # TODO: this grokker doesn't support layers yet
-
-    def execute(self, factory, method, config, context, permission, **kw):
+    def execute(
+            self, factory, method, config, context, permission, layer, **kw):
         # Create a new class with a __view_name__ attribute so the
         # JSON class knows what method to call.
         method_view = type(
             factory.__name__, (factory,),
             {'__view_name__': method.__name__}
             )
-        adapts = (context, IDefaultBrowserLayer)
+        adapts = (context, layer)
         name = method.__name__
-
         config.action(
             discriminator=('adapter', adapts, interface.Interface, name),
             callable=component.provideAdapter,
@@ -167,6 +246,7 @@ class JSONGrokker(martian.MethodGrokker):
 
 
 class TraverserGrokker(martian.ClassGrokker):
+    """Grokker for subclasses of `grok.Traverser`."""
     martian.component(grok.Traverser)
     martian.directive(grok.context)
 
@@ -181,6 +261,7 @@ class TraverserGrokker(martian.ClassGrokker):
 
 
 class SiteGrokker(martian.ClassGrokker):
+    """Grokker for subclasses of `grok.Site`."""
     martian.component(grok.Site)
     martian.priority(500)
     martian.directive(grok.local_utility, name='infos')
@@ -266,6 +347,13 @@ def setupUtility(site, utility, provides, name=u'',
 
 
 class RoleGrokker(martian.ClassGrokker):
+    """Grokker for components subclassed from `grok.Role`.
+
+    Each role is registered as a global utility providing the service
+    `IRole` under its own particular name, and then granted every
+    permission named in its `grok.permission()` directive.
+
+    """
     martian.component(grok.Role)
     martian.priority(martian.priority.bind().get(PermissionGrokker()) - 1)
     martian.directive(grok.name)
@@ -299,6 +387,7 @@ class RoleGrokker(martian.ClassGrokker):
 
 
 class AnnotationGrokker(martian.ClassGrokker):
+    """Grokker for components subclassed from `grok.Annotation`."""
     martian.component(grok.Annotation)
     martian.directive(grok.context, name='adapter_context')
     martian.directive(grok.provides, get_default=default_annotation_provides)
@@ -331,6 +420,7 @@ class AnnotationGrokker(martian.ClassGrokker):
 
 
 class ApplicationGrokker(martian.ClassGrokker):
+    """Grokker for Grok application classes."""
     martian.component(grok.Application)
     martian.priority(500)
 
@@ -347,6 +437,7 @@ class ApplicationGrokker(martian.ClassGrokker):
 
 
 class IndexesGrokker(martian.InstanceGrokker):
+    """Grokker for Grok index bundles."""
     martian.component(components.IndexesClass)
 
     def grok(self, name, factory, module_info, config, **kw):
@@ -376,7 +467,21 @@ class IndexesGrokker(martian.InstanceGrokker):
 
 
 class IndexesSetupSubscriber(object):
+    """Helper that sets up indexes when their Grok site is created.
 
+    Each `grok.Indexes` class serves as an assertion that, whenever an
+    instance of its `grok.site()` is created, the given list of indexes
+    should be generated as well.  But a long period of time could elapse
+    between when the application starts (and its indexes are grokked),
+    and the moment, maybe days or weeks later, when a new instance of
+    that `grok.Site` is created.  Hence this `IndexesSetupSubscriber`:
+    it can be instantiated at grokking time with the index information,
+    and then registered with the Component Architecture as an event that
+    should be fired later, whenever the right kind of `grok.Site` is
+    instantiated.  At that point its `__call__` method is kicked off and
+    it makes sure the index catalogs get created properly.
+
+    """
     def __init__(self, catalog_name, indexes, context, module_info):
         self.catalog_name = catalog_name
         self.indexes = indexes
@@ -425,113 +530,3 @@ class IndexesSetupSubscriber(object):
         setupUtility(site, intids, IIntIds)
         return intids
 
-
-class RESTProtocolGrokker(martian.ClassGrokker):
-    martian.component(grok.RESTProtocol)
-    martian.directive(grok.layer, default=IBrowserRequest)
-    martian.directive(grok.name, get_default=default_view_name)
-
-    def execute(self, factory, config, name, layer, **kw):
-        config.action(
-            discriminator=('restprotocol', name),
-            callable=zope.component.interface.provideInterface,
-            args=(name, layer, IRESTSkinType)
-            )
-        return True
-
-class ViewletManagerGrokker(martian.ClassGrokker):
-    martian.component(grok.ViewletManager)
-    martian.directive(grok.context)
-    martian.directive(grok.layer, default=IDefaultBrowserLayer)
-    martian.directive(grok.view)
-    martian.directive(grok.name)
-
-    def grok(self, name, factory, module_info, **kw):
-        # Need to store the module info object on the view class so that it
-        # can look up the 'static' resource directory.
-        factory.module_info = module_info
-        return super(ViewletManagerGrokker, self).grok(
-            name, factory, module_info, **kw)
-
-    def execute(self, factory, config, context, layer, view, name, **kw):
-        # This will be used to support __name__ on the viewlet manager
-        factory.__view_name__ = name
-
-        # find templates
-        templates = factory.module_info.getAnnotation('grok.templates', None)
-        if templates is not None:
-            config.action(
-                discriminator=None,
-                callable=self.checkTemplates,
-                args=(templates, factory.module_info, factory)
-                )
-
-        config.action(
-            discriminator = ('viewletManager', context, layer, view, name),
-            callable = component.provideAdapter,
-            args = (factory, (context, layer, view), IViewletManager, name)
-            )
-        return True
-
-    def checkTemplates(self, templates, module_info, factory):
-        def has_render(factory):
-            return factory.render != grok.components.ViewletManager.render
-        def has_no_render(factory):
-            # always has a render method
-            return False
-        templates.checkTemplates(module_info, factory, 'viewlet manager',
-                                 has_render, has_no_render)
-
-class ViewletGrokker(martian.ClassGrokker):
-    martian.component(grok.Viewlet)
-    martian.directive(grok.context)
-    martian.directive(grok.layer, default=IDefaultBrowserLayer)
-    martian.directive(grok.view)
-    martian.directive(grok.viewletmanager)
-    martian.directive(grok.name, get_default=default_view_name)
-    martian.directive(grok.require, name='permission')
-
-    def grok(self, name, factory, module_info, **kw):
-        # Need to store the module info object on the view class so that it
-        # can look up the 'static' resource directory.
-        factory.module_info = module_info
-        return super(ViewletGrokker, self).grok(
-            name, factory, module_info, **kw)
-
-    def execute(self, factory, config,
-                context, layer, view, viewletmanager, name, permission, **kw):
-        # This will be used to support __name__ on the viewlet
-        factory.__view_name__ = name
-
-        # find templates
-        templates = factory.module_info.getAnnotation('grok.templates', None)
-        if templates is not None:
-            config.action(
-                discriminator=None,
-                callable=self.checkTemplates,
-                args=(templates, factory.module_info, factory)
-                )
-
-        config.action(
-            discriminator = ('viewlet', context, layer,
-                             view, viewletmanager, name),
-            callable = component.provideAdapter,
-            args = (factory, (context, layer, view, viewletmanager),
-                    IViewlet, name)
-            )
-
-        config.action(
-            discriminator=('protectName', factory, '__call__'),
-            callable=make_checker,
-            args=(factory, factory, permission, ['update', 'render']),
-            )
-
-        return True
-
-    def checkTemplates(self, templates, module_info, factory):
-        def has_render(factory):
-            return factory.render != grok.components.Viewlet.render
-        def has_no_render(factory):
-            return not has_render(factory)
-        templates.checkTemplates(module_info, factory, 'viewlet',
-                                 has_render, has_no_render)
